/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Pavol Vaskovic.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Andy Edmonds <aedmonds@mindspring.com>
 *  Benjamin K. Stuhl <tiriath@yahoo.com>
 *  Brent Schartung <bschartung@gmail.com>
 *  David Illsley <illsleydc@bigfoot.com>
 *  Dorando <mozilla@dorando.at>
 *  HJ van Rantwijk <bugs4HJ@netscape.net>
 *  Jens Bannmann <jens.b@web.de>
 *  Jochen <bugs@krickelkrackel.de>
 *  Martin.T.Kutschker <Martin.T.Kutschker@blackbox.net>
 *  Pavol Vaskovic <pali@pali.sk>
 *  Pekka Aleksi Knuutila <aleksi.knuutila@edu.lahti.fi>
 *  Scott R. Turner <srt@aero.org>
 *  Tim Williamson <chsman@hotmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//----------------------
//-- Global variables --
//----------------------

var mgContext = null;
var mgContextVis = false;
var mgPopupBox;
var mgContent = null;

//-----------------
//-- Preferences --
//-----------------

var mgButton = -1;
var mgLeftHanded = false;
var mgContextOnDown = false;
// mouse gesture condition (to be evaluated on event) for more details see mgReadPref.init
var mgGestureCondition = new String;
var mgGrid; // minimal gesture has to be 'grid' pixels long
var mgDelay; // delay before aborting gesture
var mgLmbGestureLimit;
var mgDragdropDelay;
var mgRockersEnabled;
var mgWheelRockers;
var mgWheelSensitivity;
var mgWheelRockersAutoRepeat;
var mgStaticRockers;
var mgStaticRockersDelay;
var mgDiagonalTolerance; // tolerance for diagonal movement. see mgProcessCoordinates()
var mgDrawTrail;
var mgStatusEnabled;
var mgStatusTimeout;
var mgStatusFgColor;

//---------------------
//-- State variables --
//---------------------

var mgGestureStartTime = 0;
var gestureInProgress = false;
var mgGestureDone = false;
var mgRockerAborted = false;
var mgPreviousSelection = null;
// old coordinates from previous gesture stroke used in mgProcessCoordinates()
var mgOldX = 0;
var mgOldY = 0;
var mgGridMoves = new Array();
var mgGesture = new Array();
var mgLocalizedGesture = new Array();
var mgString; // array holding localized stroke names
var mgLastGesture;
// mgLastGestureTime is time of last gesture stroke, compared with delay and current time
var mgLastGestureTime = 0;
var mgLastMoveTime = 0;
// timer used to abort gestures after delay, see mgProcessCoordinates()
var mgGestureTimeout;
var mgStatusTimer;
var mgAllowContextMenu = false;
var mgAutoScrollAborted = false;

var globalOnLink = false; // array that holds a list of all the links traversed during gesture
var globalOnLinkTemp;
var globalOnImage = false; // string containing an image href or false
var globalSrcEvent = false; // event which started the active gesture.
var mgLastHoveredElement = null;

//--------------------
//-- Observer class --
//--------------------

function mgObserver() {
  this.register();
}

mgObserver.prototype = {
  mgPrefTimeout   : null,
  observerService : Components.classes["@mozilla.org/observer-service;1"]
                              .getService(Components.interfaces.nsIObserverService),

  createEvt : function(what, data) {
    var mData = data.split("|");
    var evt = document.createEvent("MouseEvents");
    evt.initMouseEvent(what, 1, 1, window, 1,
                       mData[1], mData[2], 0, 0, 0, 0, 0, 0, mData[0], window);

    return evt;
  },

  observe : function(subject, topic, data) {
    // when MozGest pref was changed, we reinitialize
    // (except when sidebar settings changed)
    // use a timeout to avoid multiple inits if more than
    // one pref was changed. Not bullet proof.
    if (topic.indexOf("nsPref") == 0) {
      if (data.indexOf("sideBar") == 0 || data == "selectedTabIndex" ||
          data == "importFrom" || data == "firstStart")
        return;

      clearTimeout(mgObserver.mgPrefTimeout);
      mgObserver.mgPrefTimeout = setTimeout(mozgestInit, 200);
    }

    if (topic == "mozgestGetBaseWin") {
      if (mgCommon.winWatcher.activeWindow != window)
        return;

      mgMouseService.setBaseWin(mgGetBaseWin());
    }

    if (topic == "mozgestButtonUp") {
      if (mgCommon.winWatcher.activeWindow != window)
        return;

      try {
        mgEndGesture(this.createEvt("mouseup", data), true);
      }
      catch (e) {}
    }

    if (topic == "mozgestRockerDown") {
      if (mgCommon.winWatcher.activeWindow != window)
        return;

      try {
        mgStartGesture(this.createEvt("mousedown", data));
      }
      catch (e) {}
    }
  },

  register : function() {
    mgPref.QueryInterface(Components.interfaces.nsIPrefBranchInternal)
          .addObserver("", this, false);

    if (mgMouseEvents) {
      this.observerService.addObserver(this, "mozgestGetBaseWin", false);
      this.observerService.addObserver(this, "mozgestRockerDown", false);
      this.observerService.addObserver(this, "mozgestButtonUp", false);
    }
  },

  unregister : function() {
    mgPref.QueryInterface(Components.interfaces.nsIPrefBranchInternal)
          .removeObserver("", this);

    if (mgMouseEvents) {
      this.observerService.removeObserver(this, "mozgestGetBaseWin");
      this.observerService.removeObserver(this, "mozgestRockerDown");
      this.observerService.removeObserver(this, "mozgestButtonUp");
    }
  }
}

//--------------------------------
//-- Startup/shutdown functions --
//--------------------------------
function mgStartup(e) {
  window.removeEventListener("load", mgStartup, false);
  mgDefaultPrefs.init();
  mgComponentLoader.initMouseService(true);

  // Hack! get our keyset
  var mgKeySet = document.getElementById("openMozgestSettings");
  if (!mgKeySet) {
    mgKeyset = document.getElementsByAttribute("id", "openMozgestSettings")[0].parentNode;
    document.documentElement.appendChild(mgKeyset);
  }

  //this comes from windowTypes.js
  mgWindowTypes.init();
  mgWindowType = mgWindowTypes.getWindowType(document.location);
  mgWindowTypes.setIt();

  if (mgWindowType) {
    mgCommon.dump("MozGest: ---- Startup ----\n");
    // init our observer
    window.mozgestObserver = new mgObserver();
    window.addEventListener("unload", mgShutdown, false);
    mgLoadStrings();
    mozgestInit();
  }
}

function mgShutdown(e) {
  mgCommon.dump("MozGest: ---- Shutdown ----\n");
  window.mozgestObserver.unregister();
  mgRemoveWindowWatch();
  // Remove this event listener after all other listeners have completed
  window.setTimeout("window.removeEventListener(\"unload\", mgShutdown, false);", 0);
}

function mozgestInit() {
  // read prefs (or set default for each missing setting)
  mgReadPref.init();
  mgStatusSetter.firstChild.setAttribute("style", "color:"+mgStatusFgColor+";")
  // turn gestures recognition on/off
  if (mgReadPref.get(mgWindowType))
    mgAddWindowWatch();
  else
    mgRemoveWindowWatch();


  try {
    mgPref.getBoolPref("firstStart");
  }
  catch (e) {
    mgPref.setBoolPref("firstStart", true);
    mgPref.setIntPref("selectedTabIndex", 3);
    setTimeout("mgCommon.openDialog('chrome://mozgest/content/pref/pref-mozgest.xul')", 500);
  }
}

function mgLoadStrings() {
  // load localized strings and cache often used strings
  mgString = new Array();
  mgString["R"] = mgGetString("abbreviation.right");
  mgString["L"] = mgGetString("abbreviation.left");
  mgString["U"] = mgGetString("abbreviation.up");
  mgString["D"] = mgGetString("abbreviation.down");
  mgString["1"] = mgGetString("abbreviation.d1");
  mgString["3"] = mgGetString("abbreviation.d3");
  mgString["7"] = mgGetString("abbreviation.d7");
  mgString["9"] = mgGetString("abbreviation.d9");
  mgString["gesture"] = mgGetString("g.gesture");
}

function mgAddWindowWatch() {
  mgCommon.dump("MozGest: ---- Get Contentarea ---\n");

  if (document.documentElement.getAttribute("windowtype") == "navigator:browser")
    mgContent = mgGetContentArea().mPanelContainer;
  else
    mgContent = mgGetContentArea();

  mgRemoveWindowWatch();
  mgCommon.dump("MozGest: Adding Listeners\n");

  mgContent.addEventListener("mousedown", mgStartGesture, false);
  mgContent.addEventListener("draggesture", mgOnDragGestureEvent, true);

  if (mgWheelRockers && mgRockersEnabled)
    mgContent.addEventListener("DOMMouseScroll", mgMousewheelHandler, false);

  window.addEventListener("mouseup", mgEndGesture, false);
  window.addEventListener("keydown", mgAllowContextByKeyPress, true);
  window.addEventListener("mousemove", mgProcessCoordinates, true);
  mgContent.addEventListener("contextmenu", mgContextMenuListener, true);
  mgContent.addEventListener("click", mgOnClickHandler, true);
  mgContent.addEventListener("dblclick", mgOnClickHandler, true);
}

function mgRemoveWindowWatch() {
  mgCommon.dump("MozGest: Removing Listeners\n");
  mgContent.removeEventListener("mousedown", mgStartGesture, false);
  mgContent.removeEventListener("draggesture", mgOnDragGestureEvent, true);
  mgContent.removeEventListener("DOMMouseScroll", mgMousewheelHandler, false)
  mgContent.removeEventListener("contextmenu", mgContextMenuListener, true)
  window.removeEventListener("mouseup", mgEndGesture, false);
  window.removeEventListener("keydown", mgAllowContextByKeyPress, true);
  window.removeEventListener("mousemove", mgProcessCoordinates, true);
  mgContent.removeEventListener("click", mgOnClickHandler, true);
  mgContent.removeEventListener("dblclick", mgOnClickHandler, true);
}

//-----------------------
//-- Integration hooks --
//-----------------------
// (everything needed for co-existing with or disabling regular app functions)

function mgContextMenuListener() {
  addEventListener("popupshowing", mgDisableContextMenu, true);
  setTimeout("removeEventListener('popupshowing', mgDisableContextMenu, true)", 5);
}

function mgDisableContextMenu(e) {
  mgContext=e.originalTarget;

  if (!mgAllowContextMenu)
    e.preventDefault();
}

function mgAllowContextByKeyPress(e) {
  mgAllowContextMenu = true;
  mgGestureDone = false;
}

function mgOnClickHandler(e) {
  if (mgGestureDone) {
    mgCommon.dump("MozGest: Stopping " + e.type + " event caused by last gesture.\n");
    e.preventDefault();
    e.stopPropagation();
    return false;
  }

  var node = e.originalTarget;
  if (!node.getAttribute) return null;
  if (!node.hasAttribute("href")) return null;
  var tagname = node.nodeName.toLowerCase();
  var href = node.getAttribute("href");
  if (tagname == "a" && href.indexOf("mozgest://") == 0 && e.button < 2) {
    mgCommon.dump("MozGest: Link click " + href + "\n");
    mgStorage.processURL(href, node.ownerDocument.location);
    e.stopPropagation();
    e.preventDefault();
  }
  return null;
}

function mgOnDragGestureEvent(e) {
  var draggedElement = e.originalTarget.nodeName.toUpperCase();
  if (gestureInProgress && draggedElement != "HTML") {
    if (mgLastMoveTime-mgGestureStartTime > mgDragdropDelay) {
      mgCommon.dump("MozGest: '" + draggedElement
           + "' drag encountered, canceling gesture\n");
      mgEndGesture("drag");
    } else {
      mgCommon.dump("MozGest: canceling '" + draggedElement
           + "' drag event, continuing gesture\n");
      e.stopPropagation();
    }
  }
  if (mgGestureDone)
    e.stopPropagation();
}

//---------------------------------
//-- The gesture recognizer core --
//---------------------------------
function mgProcessCoordinates(e) {
  if (e.screenX < mgOldX -25 || e.screenX > mgOldX +25 ||
      e.screenY < mgOldY -25 || e.screenY > mgOldY +25)
    mgRockerAborted = true;

  if (!gestureInProgress)
    return;

  var now = (new Date).getTime();

  if (mgButton == 1 && !mgAutoScrollAborted &&
      (e.screenX != mgOldX || e.screenY != mgOldY)) {
     mgAutoScrollAborted = true;
     if (mgStopAutoScroll())
       return;
  }

  if (mgDrawTrail && !mgNativeTrails && !mgTrails.noContent)
    mgTrails.draw(e.screenX, e.screenY, e);

  var g = mgGesture.join("");
  if (mgGestureCondition == "e.button==0" && (g == "L" || g == "R")
      && (now - mgGestureStartTime) > mgLmbGestureLimit) {
    mgEndGesture(null);
  }

  var x_dir = e.screenX - mgOldX;
  var y_dir = e.screenY - mgOldY;
  var x = Math.abs(x_dir);
  var y = Math.abs(y_dir);

  if (x >= mgGrid/2 || y >= mgGrid/2) { // process each half-grid
    // diagonal movement:
    // mgDiagonalTolerance = 75 means that a movement is recognized as diagonal when
    // x/y or y/x is between 0.25 and 1
    if ( mgDiagonalTolerance != 0 && ( (x/y >= (1-mgDiagonalTolerance/100) && x/y <= 1)
        || (y/x >= (1-mgDiagonalTolerance/100) && y/x <= 1) ) ) {

      if (x_dir < 0 && y_dir > 0)
        mgPush("1");
      else if (x_dir > 0 && y_dir > 0)
        mgPush("3");
      else if (x_dir < 0 && y_dir < 0)
        mgPush("7");
      else if (x_dir > 0 && y_dir < 0)
        mgPush("9");
    }
    // horizontal move:
    else if (x > y) {
      if (x_dir > 0)
        mgPush("R");
      else if (x_dir < 0)
        mgPush("L");
    }
    // vertical move:
    else if (x < y) {
      if (y_dir > 0)
        mgPush("D");
      else if (y_dir < 0)
        mgPush("U");
    }

    if (mgStatusEnabled) {
      clearTimeout(mgStatusTimer);
      var statusText = mgString["gesture"] + " " + mgLocalizedGesture;
      var nameTemp = mgGetMapping(mgGesture.join(""))[1];

      if (nameTemp)
        statusText += " (" + nameTemp + ")";

      mgStatusSetter.firstChild.setAttribute("value", statusText);

      if (mgStatusSetter.hidden) {
        mgStatusSetter.width = mgStatusSetter.parentNode.boxObject.width/2;
        mgStatusSetter.firstChild.width = mgStatusSetter.width-10;
        mgStatusSetter.hidden = false;
      }
    }

    mgLastGestureTime = now;

    if (mgGestureTimeout)
      window.clearTimeout(mgGestureTimeout);

    mgGestureTimeout = window.setTimeout("mgEndGesture(null);", mgDelay);

    mgOldX = e.screenX;
    mgOldY = e.screenY;
  }
  mgLastMoveTime = now;
  mgExamineHoveredElement(e.originalTarget);
}

function mgPush(code) {
  mgGridMoves.push(code);
  if (mgGridMoves.length == 2) {
    if (mgGridMoves[0] == mgGridMoves[1] || mgGesture.length == 0)
      mgDoPush(mgGridMoves.pop());

    mgGridMoves.length = 0;
  }
}

function mgDoPush(code) {
  if (mgGesture[mgGesture.length-1] != code) {
    mgGesture.push(code);
    mgLocalizedGesture.push(mgString[code]);
  }
  if (mgGesture.length == 1)
    mgResetRocker();
}

function mgExamineHoveredElement(node, inContentCheck) {
  var imgNode = node;
  var CI = Components.interfaces;

  if (inContentCheck) {
    if (mgMouseEvents && mgRockerCode.length > 1)
      return true;

    globalOnLink = globalOnImage = false;
    var checkNode = node;

    for (checkNode; checkNode; checkNode = checkNode.parentNode) {
      var n = checkNode.nodeName.toLowerCase();

      if (n == "scrollbar" || n == "select" ||
          n == "input" || n == "textarea" ||
          n == "textbox" || n == "object" || n == "embed")
        return false;
    }
  }

  for (node; node; node = node.parentNode) {
    if (node instanceof CI.nsIDOMHTMLAnchorElement ||
        node instanceof CI.nsIDOMHTMLAreaElement ||
        node instanceof CI.nsIDOMHTMLLinkElement) {

      if (!globalOnLink) {
        globalOnLink = new Array();
        globalOnLinkTemp = new Array();
      }

      if (!(node.href in globalOnLinkTemp)) {
        globalOnLinkTemp[node.href] = node.href;
        globalOnLink.push(node);
        mgCommon.dump("Mozgest: hovered link '" + node.href + "'\n");
      }
      break;
    }
  }

  for (imgNode; imgNode; imgNode = imgNode.parentNode) {
    if (!globalOnImage && imgNode instanceof CI.nsIDOMHTMLImageElement) {
      globalOnImage = imgNode;
      mgCommon.dump("Mozgest: found image.\n");
      break;
    }
  }
  return true;
}

function mgStartGesture(e) {
  mgReleaseRocker(e);
  mgAllowContextMenu = false;
  mgGestureDone = false;
  mgOldX = e.screenX;
  mgOldY = e.screenY;

  if (mgExamineHoveredElement(e.originalTarget, true) && !mgCheckForRocker(e)) {
    globalSrcEvent = e;

    if (!gestureInProgress && eval(mgGestureCondition)) {
      // check if it's possible to draw trails
      if (mgDrawTrail)
        mgTrails.shouldDrawTrail(e);

      gestureInProgress = true;
      mgGestureStartTime = mgLastGestureTime = (new Date).getTime();
      mgLastMoveTime = mgGesture.length = mgLocalizedGesture.length = 0;;

      var sel = mgGetSelection();
      if (mgButton == 0 && sel.rangeCount > 0)
        mgPreviousSelection = sel.getRangeAt(0);
    }
  }
}

function mgEndGesture(e, synth) {
  mgReleaseRocker(e);
  mgRockerAborted = false;
  mgAutoScrollAborted = false;
  clearTimeout(mgStatusTimer);

  if (gestureInProgress) {
    gestureInProgress = mgGestureStartTime = false;
    if (!mgNativeTrails && mgButton == 0)
      window.setTimeout("mgTrails.endTrail()", 1);
    else
      mgTrails.endTrail();

    window.clearTimeout(mgGestureTimeout);
    mgLastGesture = mgGesture.join("");

    // check if we've been called via a timeout; see mgProcessCoordinates()
    if (e == null) {
      mgCommon.dump("gesture timeouted\n");

      if (mgStatusEnabled)
        mgStatusSetter.firstChild.value = mgGetString("g.aborted");

      mgGestureDone = true;
    }
    else if (mgLastGesture != "") {
      mgGestureDone = true;
      window.setTimeout(mgFireGesture, 0, mgLastGesture);
    }
    else
      mgCommon.dump("MozGest: Gesture was too small\n");
  }

  if (mgStatusEnabled) {
    if (!mgStatusSetter.mgIsMaster)
      mgStatusTimer = setTimeout("mgStatusSetter.hidden = true", mgStatusTimeout);
    else
      mgStatusTimer = setTimeout("mgStatusSetter.firstChild.value=''", mgStatusTimeout);
  }

  if (!synth && e && !mgGestureDone && (e.button == 2 ||
      (navigator.platform.indexOf("Mac") == 0 && e.button == 0 && e.ctrlKey))) {
    mgAllowContextMenu = true;
    //==================== contextmenu on mousedown =============================
    if (navigator.platform != "Win32" && navigator.platform != "OS/2") {
      var ctBox = mgContent.boxObject;

      if (e.screenY >= ctBox.screenY &&
          e.screenY <= ctBox.screenY + ctBox.height &&
          e.screenX >= ctBox.screenX &&
          e.screenX <= ctBox.screenX + ctBox.width) {

        document.popupNode = e.target;
        mgPopupBox = mgContext.boxObject
                     .QueryInterface(Components.interfaces.nsIPopupBoxObject);

        if (mgContextVis)
          return;

        if (!mgContext.hasAttribute("mozgestWasHere")) {
          mgContext.addEventListener("popupshown", mgObserveContext, true);
          mgContext.addEventListener("popuphidden", mgObserveContext, true);
          mgContext.setAttribute("mozgestWasHere", true);
        }

        if (e.target.ownerDocument && e.target.ownerDocument == document)
          return;

        try {
          if (!mgPopupBox.openPopupAtScreen)
            mgPopupBox.showPopup(mgContext.ownerDocument.documentElement, mgContext,
                                 e.clientX, e.clientY, "context", "bottomleft", "topleft");
          else
            mgPopupBox.openPopupAtScreen(e.screenX, e.screenY, true);
        }
        catch (e) {}
      }
    }
  }
}

function mgObserveContext(e) {
  if (e.type == "popupshown") {
    if (e.target == mgContext)
      mgContextVis = true;
  }
  else if (e.target == mgContext)
    mgContextVis = false;
}

function mgGetBaseWin() {
  var mgDocShell = mgGetContentArea().docShell;

  if (mgContent.id == "output-deck")
    mgDocShell = mgGetContentArea().selectedPanel.docShell;

  return mgDocShell.QueryInterface(Components.interfaces.nsIBaseWindow);
}

//---------------------------
//-- Preferences interface --
//---------------------------
var mgReadPref = {
  init : function() {
    var g = this.get;
    mgGrid = g("grid");
    mgDelay = g("delay");
    mgContextOnDown = g("contextOnDown");
    mgLmbGestureLimit = g("lmbGestureLimit");
    mgDragdropDelay = g("dragdropDelay");
    mgRockersEnabled = g("enableRockers");
    mgWheelRockers = g("wheelRockers");
    mgWheelSensitivity = g("wheelSensitivity");
    mgWheelRockersAutoRepeat = g("wheelRockersAutoRepeat");
    mgStaticRockers = g("staticRockers");
    mgStaticRockersDelay = g("staticRockersDelay");
    mgDiagonalTolerance = g("diagonalTolerance");
    mgButton = g("mousebutton");
    mgLeftHanded = g("lefthanded");
    if (mgLeftHanded)
      mgButton = Math.abs(mgButton - 2);

    // mgGestureCondition is a boolean expression consisting of event object values
    mgGestureCondition = "e.button==" + mgButton;
    if (g("modifier.ctrl"))
      mgGestureCondition += " && e.ctrlKey";
    if (g("modifier.alt"))
      mgGestureCondition += " && e.altKey";
    if (g("modifier.shift"))
      mgGestureCondition += " && e.shiftKey";
    if (navigator.platform == "MacPPC" && g("modifier.meta"))
      mgGestureCondition += " && e.metaKey";

    mgDrawTrail = g("trails.enabled");
    mgTrails.width = g("trails.width");
    mgTrails.interval = g("trails.interval");
    mgTrails.color = g("trails.color");
    mgStatusEnabled = g("status.enabled");
    mgStatusTimeout = g("status.timeout");
    mgStatusFgColor = g("status.fgColor");
    g("sideBarSort");
    g("sideBarToolTip");
    g("importFrom");
    g("selectedTabIndex");
  },

  get : function(name) {
    if (!name || !(name in mgDefaultPrefs.prefs))
      return null;

    var mgDefault = mgDefaultPrefs.prefs[name];
    var type;

    switch(typeof mgDefault[0]) {
      case "number":
        type = "Int"
        break;
      case "boolean":
        type = "Bool";
        break;
      case "string":
        type = "Char";
        break;
      default:
        break;
    }

    var notfound = false;
    var value;

    try {
      value = eval("mgPref.get" + type + "Pref('" + name + "');");
    }
    catch (e) {
      notfound = true;
    }
    if (notfound || (type == "Int" && (value < mgDefault[1] || value > mgDefault[2]))) {
      var val = (type == "Char") ? ("'" + mgDefault[0] + "'") : mgDefault[0];
      value = eval("mgPref.set" + type + "Pref('" + name + "', " + val + ");");
    }
    return value;
  }
}